% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/CoronaVirus_Disease_2019.R
\name{CoronaVirus_Disease_2019_prevalence}
\alias{CoronaVirus_Disease_2019_prevalence}
\title{Who should be inspected?}
\usage{
CoronaVirus_Disease_2019_prevalence(pre, se, sp)
}
\arguments{
\item{pre}{Prevalence of population}

\item{se}{Sensitivity of a diagnostic test}

\item{sp}{Specificity of a diagnostic test}
}
\value{
same as \code{ \link{CoronaVirus_Disease_2019}()}

\deqn{Prob(Truth = diseased | Diagnosis = Positive) = \frac{Se\times pre}{Se \times pre + (1-pre)\times(1-sp)}   }



where we denotes the \emph{conditional probability measure}
of an event \eqn{A}
given the assumed occurrence of \eqn{G}
as an usual manner

 \deqn{P(A|G):= \frac{P(A \cap G)}{P(G)}. }
}
\description{
Even if we test all people, the result is true with very low probabilties.
}
\details{
--------------------------------------------------------------------------
\tabular{llll}{

 Diagnosis \ truth   \tab \strong{ Diseased } \tab \strong{   Non-diseased}  \cr
    -----------------------\tab -----------------------   \tab ------------- \cr
 Positive   \tab   se*n     \tab  \eqn{(N-n)(1-sp)}                 \cr
 Negative   \tab   (1-se)*n \tab  \eqn{(N-n)sp}             \cr
    -----------------------\tab -----------------------   \tab ------------- \cr
        \tab   n \tab  \eqn{N-n}      \cr
 }
-------------------------------------------------------------------------


For example,

  if prevalence is 0.0001,

   population is 10000,

specificity = 0.8,

sensitivity = 0.9,


  then the table is the following.

  We can calculates the probability of the event that
  positive-diagnosis correctly detects the diseased patient
  is

  \deqn{  \frac{9}{1998 + 9} = 9/(1998+9) = 0.00448 }


--------------------------------------------------------------------------
\tabular{llll}{

 Diagnosis \ truth   \tab \strong{ Diseased } \tab \strong{   Non-diseased}  \cr
    -----------------------\tab -----------------------   \tab ------------- \cr
 Positive   \tab   9     \tab   1998                \cr
 Negative   \tab   1 \tab   7992            \cr
    -----------------------\tab -----------------------   \tab ------------- \cr
        \tab   \eqn{n = 10} \tab  \eqn{N-n=10000-10}      \cr
 }
-------------------------------------------------------------------------
}
\examples{

CoronaVirus_Disease_2019_prevalence(0.0001, 0.9,0.8)
CoronaVirus_Disease_2019_prevalence(0.03,0.9,0.8)
CoronaVirus_Disease_2019_prevalence(0.3,0.9,0.8)



#========================================================================================
#  If Sensitivity and Specificity is larger, then, the probability is also larger
#========================================================================================


x <- stats::runif(1111,0,1)
y <- CoronaVirus_Disease_2019_prevalence(0.1,x,x)

dark_theme(4)
plot(x,y)








x <- stats::runif(1111,0,1)
y <- CoronaVirus_Disease_2019_prevalence(0.01,x,x)

dark_theme(4)
plot(x,y)






x <- stats::runif(1111,0,1)
y <- CoronaVirus_Disease_2019_prevalence(0.001,x,x)

dark_theme(4)
plot(x,y)





#========================================================================================
#  linear case:
#
#   If prevalence is 0.5
#       and sensitivity = specificity
#   then, the probability is exactly same as sensitivity = specificity
#
#========================================================================================



x <- stats::runif(1111,0,1)
y <- CoronaVirus_Disease_2019_prevalence(0.5,x,x)

dark_theme(4)
plot(x,y)


sum(x==y)==length(x)

# Because the last is true, the probablity is same as sensitivity
# when the prevalence is 0.5.












#========================================================================================
#  If the prevalence is larger, then, the probability is also larger
#========================================================================================



x <- stats::runif(1111,0,1)
y <- CoronaVirus_Disease_2019_prevalence(x,0.9,0.9)

dark_theme(4)
plot(x,y)







}
\seealso{
\code{ \link{CoronaVirus_Disease_2019}()}
}
